/*
 * $Id: avrcore.h,v 1.18 2009/02/24 19:54:32 joerg_wunsch Exp $
 *
 ****************************************************************************
 *
 * simulavr - A simulator for the Atmel AVR family of microcontrollers.
 * Copyright (C) 2001, 2002, 2003, 2004  Theodore A. Roth
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 ****************************************************************************
 */

#ifndef SIM_AVRCORE_H
#define SIM_AVRCORE_H


#define RTAVR

#include <stdio.h>
#include <malloc.h>
#include <errno.h>
#include <fcntl.h>

#include <stdlib.h>

#if !defined(__linux__) 
typedef char  int8_t;
typedef short int16_t;
typedef long  int32_t;
#endif

typedef unsigned char  uint8_t;
typedef unsigned short uint16_t;
typedef unsigned long  uint32_t;

#define AvrCore void
extern uint8_t avr_core_mem_read (AvrCore *core, int addr);
extern void avr_core_mem_write (AvrCore *core, int addr, uint8_t val);
extern uint8_t avr_core_sreg_get (AvrCore *core);
extern void avr_core_sreg_set (AvrCore *core, uint8_t v);
extern int avr_core_sreg_get_bit (AvrCore *core, int b);
extern void avr_core_sreg_set_bit (AvrCore *core, int b, int v);
extern uint8_t avr_core_gpwr_get (AvrCore *core, int reg);
extern void avr_core_gpwr_set (AvrCore *core, int reg, uint8_t val, int as_idx);
extern void avr_core_io_write (AvrCore *core, int reg, uint8_t val);
extern uint8_t avr_core_io_read (AvrCore *core, int reg);
extern uint32_t avr_core_stack_pop (AvrCore *core, int bytes);
extern void avr_core_stack_push (AvrCore *core, int bytes, uint32_t val);
extern int32_t avr_core_PC_size (AvrCore *core);
extern int32_t avr_core_PC_get (AvrCore *core);
extern void avr_core_PC_set (AvrCore *core, int32_t val);
extern void avr_core_PC_incr (AvrCore *core, int val);
extern void avr_core_inst_CKS_set (AvrCore *core, int val);
//extern

#define avr_warning(fmt, args...) \
    private_avr_warning(__FILE__, __LINE__, fmt, ## args)

#define avr_error(fmt, args...) \
    private_avr_error(__FILE__, __LINE__, fmt, ## args)

extern void private_avr_warning (char *file, int line, char *fmt, ...);
extern void private_avr_error (char *file, int line, char *fmt, ...);

static inline uint8_t
set_bit_in_byte (uint8_t src, int bit, int val)
{
    return ((src & ~(1 << bit)) | ((val != 0) << bit));
}

static inline uint16_t
set_bit_in_word (uint16_t src, int bit, int val)
{
    return ((src & ~(1 << bit)) | ((val != 0) << bit));
}

typedef union _uSREG uSREG;

union _uSREG
{
    uint8_t reg;
    struct
    {
        uint8_t C:1;
        uint8_t Z:1;
        uint8_t N:1;
        uint8_t V:1;
        uint8_t S:1;
        uint8_t H:1;
        uint8_t T:1;
        uint8_t I:1;
    } bit;
};

enum _sreg_bits
{
    SREG_C = 0,
    SREG_Z = 1,
    SREG_N = 2,
    SREG_V = 3,
    SREG_S = 4,
    SREG_H = 5,
    SREG_T = 6,
    SREG_I = 7
};

extern int flash_size;
extern uint16_t flash[];
extern int ram_size;
extern uint8_t ram[];

extern uSREG sreg;
extern uint16_t SP;
extern uint16_t PC;
extern uint8_t gpr[16];

extern int sleep_occ;
extern int wdr_occ;

extern int log_done;

#endif /* SIM_AVRCORE_H */
