`timescale 10ps / 10ps
`define GTK // use VCD

`define MODULE	tb_spi

module `MODULE;

   reg CLK;
   reg SCLK;
   reg CS;
   reg SCK;
   wire MOSI;

   reg [7:0] spi_data;
   reg spi_in;
   assign MOSI = spi_data[7];
  reg [7:0] LOAD_DATA;

  wire [15:0] ADDR;
  wire [7:0] STORE_DATA;

  reg [7:0] CLK_COUNT;

  isp ISP (.CLK(CLK)
		, .MOSI(MOSI)
                , .SCK(SCLK)
                , .CS(CS)
                , .MISO(MISO)

                , .ISP_STORE(ISP_STORE)
                , .ISP_LOAD(ISP_LOAD)
                , .ISP_ADDR(ADDR)
                , .ISP_STORE_DATA(STORE_DATA)
                , .ISP_LOAD_DATA(LOAD_DATA)
		);

   parameter CLKP = 5000;        //  50ns
   parameter CLKH = CLKP/2;      //
   parameter CLKL = CLKP-CLKH;   //
   
   parameter SCLKP = 21000;        //  210ns
   parameter SCLKH = SCLKP/2;      //
   parameter SCLKL = SCLKP-CLKH;   //
   
   always begin
      #CLKH CLK = 0;
      #CLKL CLK = 1;
   end

   always begin
      #SCLKH SCLK = 0;
      #SCLKL SCLK = 1;
   end

`ifdef GTK
   initial begin
      $dumpfile("tb.vcd");
      $dumpvars(0,`MODULE);
   end
`endif

// SSTPR     0110 100a  PR[a]
//           XXXX XXXX  XXXX XXXX
// 
// SLD       0010 0I00  XXXX XXXX    ( if I == 1 , PR = PR +1 )
//           XXXX XXXX  load_data
// 
// SST       0110 0I00  store_data   ( if I == 1 , PR = PR +1 )
//           XXXX XXXX  XXXX XXXX
   initial begin : test
      sys_reset();

      LOAD_DATA = 8'h5a;
      spi_out(8'h68, 8'h40); // SSTPR(lo)
      spi_out(8'h69, 8'h01); // SSTPR(hi)

      spi_out(8'h60, 8'h81); // SST(0x140)
      spi_out(8'h64, 8'h18); // SST(0x140++)
      spi_out(8'h64, 8'hAA); // SST(0x141++)
      spi_out(8'h64, 8'hBB); // SST(0x141++)
      spi_out(8'h64, 8'hCC); // SST(0x141++)
      LOAD_DATA = 8'h11;
      spi_out(8'h68, 8'h50); // SSTPR(lo)
      spi_out(8'h69, 8'h01); // SSTPR(hi)
      spi_out(8'h24, 8'hFF); // SLD(0x150)
      spi_out(8'h24, 8'hFF); // SLD(0x150++)
      LOAD_DATA = 8'h22;
      spi_out(8'h24, 8'hFF); // SLD(0x151++)
      LOAD_DATA = 8'h33;
      spi_out(8'h24, 8'hFF); // SLD(0x152++)
      $display("*** Simulation Complete! ***");
      $finish;
   end

   task sys_reset;
      begin
         CLK = 0;
         SCLK = 0;
	 CS = 0;
	 @(negedge SCLK);
         @(posedge SCLK);
      end
   endtask // sys_reset
   
   task spi_out;
      input [7:0] cmd;
      input [7:0] data;
      begin
	
        @(negedge SCLK);
	 #1 CS = 1;
	 spi_data = cmd ;
        CLK_COUNT = 0;
        while ( CLK_COUNT < 8 )
        begin
           @(posedge SCLK);
	   spi_in = MISO;
           @(negedge SCLK);
	   spi_data = { spi_data[6:0] , spi_in };
           CLK_COUNT = CLK_COUNT + 1;
        end
	 spi_data = data ;
        CLK_COUNT = 0;
        while ( CLK_COUNT < 8 )
         begin
           @(posedge SCLK);
	   spi_in = MISO;
           @(negedge SCLK);
	   spi_data = { spi_data[6:0] , spi_in };
           CLK_COUNT = CLK_COUNT + 1;
         end
	 #1 CS = 0;
      end
   endtask

endmodule // test
