/* Copyright (c) 2010 Atmel Corporation
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.

   * Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in
     the documentation and/or other materials provided with the
     distribution.

   * Neither the name of the copyright holders nor the names of
     contributors may be used to endorse or promote products derived
     from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE. */

/* $Id$ */

/* avr/iort.h - definitions for RTAVR */

/* This file should only be included from <avr/io.h>, never directly. */

#ifndef _AVR_IO_H_
#  error "Include <avr/io.h> instead of this file."
#endif
#ifdef HAVE_LOCAL_DEFS
#include "rtavr_defs.h"
#else
#include <avr/rtavr40_defs.h>
#endif

#ifndef _AVR_IOXXX_H_
#  define _AVR_IOXXX_H_ "iort.h"
#else
#  error "Attempt to include more than one <avr/ioXXX.h> file."
#endif 


#ifndef _RTAVR_rtavr40_H_
#define _RTAVR_rtavr40_H_ 1


/* Registers and associated bit numbers. */

#ifdef IOR_HAVE_PORTA
#define PINA _SFR_IO8(__PORTA_ADDR)
#define PINA0 0
#define PINA1 1
#define PINA2 2
#define PINA3 3
#define PINA4 4
#define PINA5 5
#define PINA6 6
#define PINA7 7

#define DDRA _SFR_IO8(__PORTA_ADDR+1)
#define DDA0 0
#define DDA1 1
#define DDA2 2
#define DDA3 3
#define DDA4 4
#define DDA5 5
#define DDA6 6
#define DDA7 7

#define PORTA _SFR_IO8(__PORTA_ADDR+2)
#define PORTA0 0
#define PORTA1 1
#define PORTA2 2
#define PORTA3 3
#define PORTA4 4
#define PORTA5 5
#define PORTA6 6
#define PORTA7 7

#ifdef IOR_PORT_HAVE_PUE
#define PUEA _SFR_IO8(__PORTA_ADDR+3)
#define PUEA0 0
#define PUEA1 1
#define PUEA2 2
#define PUEA3 3
#define PUEA4 4
#define PUEA5 5
#define PUEA6 6
#define PUEA7 7
#endif
#endif // IOR_HAVE_PORTA

#ifdef IOR_HAVE_PORTB
#define PINB _SFR_IO8(__PORTB_ADDR)
#define PINB0 0
#define PINB1 1
#define PINB2 2
#define PINB3 3
#define PINB4 4
#define PINB5 5
#define PINB6 6
#define PINB7 7

#define DDRB _SFR_IO8(__PORTB_ADDR+1)
#define DDB0 0
#define DDB1 1
#define DDB2 2
#define DDB3 3
#define DDB4 4
#define DDB5 5
#define DDB6 6
#define DDB7 7

#define PORTB _SFR_IO8(__PORTB_ADDR+2)
#define PORTB0 0
#define PORTB1 1
#define PORTB2 2
#define PORTB3 3
#define PORTB4 4
#define PORTB5 5
#define PORTB6 6
#define PORTB7 7

#ifdef IOR_PORT_HAVE_PUE
#define PUEB _SFR_IO8(__PORTB_ADDR+3)
#define PUEB0 0
#define PUEB1 1
#define PUEB2 2
#define PUEB3 3
#define PUEB4 4
#define PUEB5 5
#define PUEB6 6
#define PUEB7 7
#endif
#endif // IOR_HAVE_PORTB

#define GIFR _SFR_IO8(__GIFR_ADDR)
#define INTF0 0
#define INTF1 1
//#define INTF2 2
//#define INTF3 3

#define GIMSK _SFR_IO8(__GIFR_ADDR+1)
#define INT0 0
#ifdef IOR_HAVE_INT1
#define INT1 1
#endif
//#define INT2 2
//#define INT3 3

#ifdef IOR_HAVE_USART
/* USART Baud Rate Register Low UBBRL[7:0] */
#define UBRRL   _SFR_IO8(__USART_ADDR)
/* ATtiny USART Baud Rate Register High UBBRH[11:8] */
#define UBRRH   _SFR_IO8(__USART_ADDR+1)

/* ATtiny USART Control and Status Register C UCSRC */
#define UCSRC   _SFR_IO8(__USART_ADDR+2)

#define UMSEL   6
#define UPM1    5
#define UPM0    4
#define USBS    3
#define UCSZ1   2
#define UCSZ0   1
#define UCPOL   0

/* ATtiny USART Control Register UCSRB */
#define UCSRB   _SFR_IO8(__USART_ADDR+3)

#define RXCIE   7
#define TXCIE   6
#define UDRIE   5
#define RXEN    4
#define TXEN    3
#define UCSZ2   2
#define RXB8    1
#define TXB8    0

/* ATtiny USART Status Register UCSRA */
#define UCSRA   _SFR_IO8(__USART_ADDR+4)

#define RXC     7
#define TXC     6
#define UDRE    5
#define FE      4
#define DOR     3
#define UPE     2
#define U2X     1
#define MPCM    0

/* USART I/O Data Register UBR or RXB[7:0], TXB[7:0] */
#define UDR     _SFR_IO8(__USART_ADDR+5)
#define RXB     _SFR_IO8(__USART_ADDR+5)
#define TXB     _SFR_IO8(__USART_ADDR+5)
#endif // IOR_HAVE_USART

#ifdef IOR_HAVE_TIM0
#define OCR0B _SFR_IO8(__TIM0_ADDR)
#define OCR0B_0 0
#define OCR0B_1 1
#define OCR0B_2 2
#define OCR0B_3 3
#define OCR0B_4 4
#define OCR0B_5 5
#define OCR0B_6 6
#define OCR0B_7 7

#define OCR0A _SFR_IO8(__TIM0_ADDR+1)
#define OCR0_0 0
#define OCR0_1 1
#define OCR0_2 2
#define OCR0_3 3
#define OCR0_4 4
#define OCR0_5 5
#define OCR0_6 6
#define OCR0_7 7

#define TCNT0 _SFR_IO8(__TIM0_ADDR+2)
#define TCNT0_0 0
#define TCNT0_1 1
#define TCNT0_2 2
#define TCNT0_3 3
#define TCNT0_4 4
#define TCNT0_5 5
#define TCNT0_6 6
#define TCNT0_7 7

#define TCCR0B _SFR_IO8(__TIM0_ADDR+3)
#define CS00 0
#define CS01 1
#define CS02 2
#define WGM02 3
#define PSR 4
#define TSM 5
#define FOC0B 6
#define FOC0A 7

#define TCCR0A _SFR_IO8(__TIM0_ADDR+4)
#define WGM00 0
#define WGM01 1
#define COM0B0 4
#define COM0B1 5
#define COM0A0 6
#define COM0A1 7
#endif // IOR_HAVE_TIM0

#ifdef IOR_HAVE_PORTC
#define PINC _SFR_IO8(__PORTC_ADDR)
#define PINC0 0
#define PINC1 1
#define PINC2 2
#define PINC3 3
#define PINC4 4
#define PINC5 5
#define PINC6 6
#define PINC7 7

#define DDRC _SFR_IO8(__PORTC_ADDR+1)
#define DDC0 0
#define DDC1 1
#define DDC2 2
#define DDC3 3
#define DDC4 4
#define DDC5 5
#define DDC6 6
#define DDC7 7

#define PORTC _SFR_IO8(__PORTC_ADDR+2)
#define PORTC0 0
#define PORTC1 1
#define PORTC2 2
#define PORTC3 3
#define PORTC4 4
#define PORTC5 5
#define PORTC6 6
#define PORTC7 7

#ifdef IOR_PORT_HAVE_PUE
#define PUEC _SFR_IO8(__PORTC_ADDR+3)
#define PUEC0 0
#define PUEC1 1
#define PUEC2 2
#define PUEC3 3
#define PUEC4 4
#define PUEC5 5
#define PUEC6 6
#define PUEC7 7
#endif
#endif // IOR_HAVE_PORTC

#ifdef IOR_HAVE_RAMWIN
#define RAMDR _SFR_IO8(__RAMWIN_ADDR)
#define RAMDR0 0
#define RAMDR1 1
#define RAMDR2 2
#define RAMDR3 3
#define RAMDR4 4
#define RAMDR5 5
#define RAMDR6 6
#define RAMDR7 7

#define RAMAR _SFR_IO8(__RAMWIN_ADDR+1)
#define RAMAR0 0
#define RAMAR1 1
#define RAMAR2 2
#define RAMAR3 3
#define RAMAR4 4
#define RAMAR5 5
#define RAMAR6 6
#define RAMAR7 7

#ifdef IOR_HAVE_RAMWIN_HA
#define RAMAR_HA _SFR_IO8(__GIFR_ADDR+1)
#define RAMAR_HA_MASK 0x0E
#define RAMAR8   1
#define RAMAR9   2
#define RAMAR10  3
#endif
#endif // IOR_HAVE_RAMWIN

#ifdef IOR_HAVE_TIM0
#define TIFR _SFR_IO8(__TIFR_ADDR)
#define TOV0 0
#define OCF0A 1
#define OCF0B 2
//#define TOV1 3
//#define OCF1A 4
//#define OCF1B 5
//#define ICF1 7

#define TIMSK _SFR_IO8(__TIFR_ADDR+1)
#define TOIE0 0
#define OCIE0A 1
#define OCIE0B 2
//#define TOIE1 3
//#define OCIE1A 4
//#define OCIE1B 5
//#define ICIE1 7
#endif // IOR_HAVE_TIM0

#ifdef IOR_HAVE_SPI
#define SPDR _SFR_IO8(__SPI_ADDR)
#define SPDR0 0
#define SPDR1 1
#define SPDR2 2
#define SPDR3 3
#define SPDR4 4
#define SPDR5 5
#define SPDR6 6
#define SPDR7 7

#define SPSR _SFR_IO8(__SPI_ADDR+1)
#define SPI2X 0
#define WCOL 6
#define SPIF 7

#define SPCR _SFR_IO8(__SPI_ADDR+2)
#define SPR0 0
#define SPR1 1
#define CPHA 2
#define CPOL 3
#define MSTR 4
#define DORD 5
#define SPE 6
#define SPIE 7
#endif // IOR_HAVE_SPI

#ifdef IOR_HAVE_NVM
#define NVMCSR _SFR_IO8(__NVM_ADDR)
#define NVMBSY 7
#define PSBBSY 6
#define CPUID  0

#define NVMCMD _SFR_IO8(__NVM_ADDR+1)
#define NVMCMD0 0
#define NVMCMD1 1
#define NVMCMD2 2
#define NVMCMD3 3
#define NVMCMD4 4
#define NVMCMD5 5
#endif // IOR_HAVE_NVM

#ifdef IOR_HAVE_CPI0
#define CPICSR0 _SFR_IO8(__CPI0_ADDR)
#define CPIDR0 _SFR_IO8(__CPI0_ADDR+1)
#endif // IOR_HAVE_CPI0
#ifdef IOR_HAVE_CPI1
#define CPICSR1 _SFR_IO8(__CPI1_ADDR)
#define CPIDR1 _SFR_IO8(__CPI1_ADDR+1)
#endif // IOR_HAVE_CPI1
#ifdef IOR_HAVE_CPI
#define CPIR		7
#define CPIE		6
#define CPIS		5
#define CPIRXS		4
#define CPITXS3		3
#define CPITXS2		2
#define CPITXS1		1
#define CPITXS0		0
#define CPITXS_FULL(x)		((x) & (1<<3))
#define CPITXS_EMPTY(x)		~((x) & (1<<0))
#endif // IOR_HAVE_CPI

#ifdef __GPIOR0_ADDR
#define GPIOR0 _SFR_IO8(__GPIOR0_ADDR)
#endif
#ifdef __GPIOR1_ADDR
#define GPIOR1 _SFR_IO8(__GPIOR1_ADDR)
#endif
#ifdef __GPIOR2_ADDR
#define GPIOR2 _SFR_IO8(__GPIOR2_ADDR)
#endif

#define MCUCR _SFR_IO8(0x3A)
//#define SE 0
//#define SM0 1
//#define SM1 2
//#define SM2 3
//#define BODS 4
//#define ISC30 0
//#define ISC31 1
//#define ISC20 2
//#define ISC21 3
#ifdef IOR_HAVE_INT1
#define ISC10 4
#define ISC11 5
#endif
#define ISC00 6
#define ISC01 7

#define _VECTOR_SIZE 2 /* Size of individual vector. */
#define _VECTORS_SIZE (19 * _VECTOR_SIZE)

/* Interrupt vectors */
/* Vector 0 is the reset vector */
/* External Interrupt Request 0 */
//#define INT0_vect_num		1
//#define INT1_vect_num		2
//#define INT2_vect_num		3
//#define INT3_vect_num		4
//#define INT0_vect		_VECTOR(INT0_vect_num)
//#define INT1_vect		_VECTOR(INT1_vect_num)
//#define INT2_vect		_VECTOR(INT2_vect_num)
//#define INT3_vect		_VECTOR(INT3_vect_num)

/* Timer/Counter0 Compare Match A */
//#define TIM0_COMPA_vect_num	11
//#define TIM0_COMPA_vect	_VECTOR(11)
/* Timer/Counter0 Compare Match B */
//#define TIM0_COMPB_vect_num	12
//#define TIM0_COMPB_vect	_VECTOR(12)
/* Timer/Counter0 Overflow */
//#define TIM0_OVF_vect_num	13
//#define TIM0_OVF_vect		_VECTOR(13)

/* Two-Wire Interface */
//#define TWI_SLAVE_vect_num    16
//#define TWI_SLAVE_vect	_VECTOR(16)

/* Serial Peripheral Interface (SPI) */
//#define SPI_vect_num		17
//#define SPI_vect		_VECTOR(17)

/* USART, Tx Complete */
//#define USART_TX_vect_num	14
//#define USART_UDRE_vect_num	15
//#define USART_RX_vect_num	18
//#define USART_TX_vect		_VECTOR(14)
/* USART Data Register Empty */
//#define USART_UDRE_vect	_VECTOR(15)
/* USART, Rx Complete */
//#define USART_RX_vect		_VECTOR(18)

/* Two-Wire Interface (TWI) */
//#define TWI_SLAVE_vect_num	16
//#define TWI_SLAVE_vect      	_VECTOR(16)

/* Constants */
#define SPM_PAGESIZE (64)
#define RAMSTART     (0x40)
#define RAMSIZE      (256)
#define RAMEND       (RAMSTART + RAMSIZE - 1)
#define XRAMSTART    (NA)
#define XRAMSIZE     (0)
#define XRAMEND      (RAMEND)
#define E2END        (0x0)
#define E2PAGESIZE   (0)
#define FLASHEND     (0xFFF)

/* Fuses */
#define FUSE_MEMORY_SIZE 0

/* Lock Bits */
#define __LOCK_BITS_EXIST

#ifdef RTAVR40_NOT_YET
#define OCR1B _SFR_IO8(0x21)
#define OCR1B0 0
#define OCR1B1 1
#define OCR1B2 2
#define OCR1B3 3
#define OCR1B4 4
#define OCR1B5 5
#define OCR1B6 6
#define OCR1B7 7

#define OCR1A _SFR_IO8(0x22)
#define OCR1A0 0
#define OCR1A1 1
#define OCR1A2 2
#define OCR1A3 3
#define OCR1A4 4
#define OCR1A5 5
#define OCR1A6 6
#define OCR1A7 7

#define TCNT1L _SFR_IO8(0x23)
#define TCNT1_0 0
#define TCNT1_1 1
#define TCNT1_2 2
#define TCNT1_3 3
#define TCNT1_4 4
#define TCNT1_5 5
#define TCNT1_6 6
#define TCNT1_7 7

#define TCCR1A _SFR_IO8(0x24)
#define CS10 0
#define CS11 1
#define CS12 2
#define CTC1 3
#define ICES1 4
#define ICNC1 5
#define ICEN1 6
#define TCW1 7

#define TCNT1H _SFR_IO8(0x27)
#define TCNT1_8 0
#define TCNT1_9 1
#define TCNT1_10 2
#define TCNT1_11 3
#define TCNT1_12 4
#define TCNT1_13 5
#define TCNT1_14 6
#define TCNT1_15 7

#define TWSD _SFR_IO8(0x28)
#define TWSD0 0
#define TWSD1 1
#define TWSD2 2
#define TWSD3 3
#define TWSD4 4
#define TWSD5 5
#define TWSD6 6
#define TWSD7 7

#define TWSAM _SFR_IO8(0x29)
#define TWAE 0
#define TWSAM1 1
#define TWSAM2 2
#define TWSAM3 3
#define TWSAM4 4
#define TWSAM5 5
#define TWSAM6 6
#define TWSAM7 7

#define TWSA _SFR_IO8(0x2A)
#define TWSA0 0
#define TWSA1 1
#define TWSA2 2
#define TWSA3 3
#define TWSA4 4
#define TWSA5 5
#define TWSA6 6
#define TWSA7 7

#define TWSSRA _SFR_IO8(0x2B)
#define TWAS 0
#define TWDIR 1
#define TWBE 2
#define TWC 3
#define TWRA 4
#define TWCH 5
#define TWASIF 6
#define TWDIF 7

#define TWSCRB _SFR_IO8(0x2C)
#define TWCMD0 0
#define TWCMD1 1
#define TWAA 2

#define TWSCRA _SFR_IO8(0x2D)
#define TWSME 0
#define TWPME 1
#define TWSIE 2
#define TWEN 3
#define TWASIE 4
#define TWDIE 5
#define TWSHE 7

#define WDTCSR _SFR_IO8(0x31)
#define WDP0 0
#define WDP1 1
#define WDP2 2
#define WDE 3
#define WDP3 5
#define WDIE 6
#define WDIF 7

#define CLKPSR _SFR_IO8(0x36)
#define CLKPS0 0
#define CLKPS1 1
#define CLKPS2 2
#define CLKPS3 3

#define CLKMSR _SFR_IO8(0x37)
#define CLKMS0 0
#define CLKMS1 1

#define RSTFLR _SFR_IO8(0x3B)
#define PORF 0
#define EXTRF 1
#define WDRF 3

#define CCP _SFR_IO8(0x3C)
#define CCP0 0
#define CCP1 1
#define CCP2 2
#define CCP3 3
#define CCP4 4
#define CCP5 5
#define CCP6 6
#define CCP7 7

#define WDT_vect_num  4
#define WDT_vect      _VECTOR(4)  /* Watchdog Time-out */
#define TIM1_CAPT_vect_num  5
#define TIM1_CAPT_vect      _VECTOR(5)  /* Timer/Counter1 Input Capture */
#define TIM1_COMPA_vect_num  6
#define TIM1_COMPA_vect      _VECTOR(6)  /*  Timer/Counter1 Compare Match A */
#define TIM1_COMPB_vect_num  7
#define TIM1_COMPB_vect      _VECTOR(7)  /*  Timer/Counter1 Compare Match B */
#define TIM1_OVF_vect_num  8
#define TIM1_OVF_vect      _VECTOR(8)  /* Timer/Counter1 Overflow */
#define ANA_COMP_vect_num  12
#define ANA_COMP_vect      _VECTOR(12)  /* Analog Comparator */
#define ADC_ADC_vect_num  13
#define ADC_ADC_vect      _VECTOR(13)  /* Conversion Complete */

/* Signature */
#define SIGNATURE_0 0x1E
#define SIGNATURE_1 0x92
#define SIGNATURE_2 0x0E

#endif /* RTAVR40_NOT_YET */
#endif /* _RTAVR_rtavr40_H_ */
