#include <stdlib.h>
#include <string.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <util/delay.h>

#define bit_set(port,bit)       { port |= (1<<(bit));}
#define bit_clear(port,bit)     { port &= ~(1<<(bit));}
#define min(a,b)                ((a)<(b) ? (a):(b))

#define nop()   __asm__ volatile ("nop" )
#define wdr()   __asm__ volatile("wdr" )
#define sleep()   __asm__ volatile("sleep" )


uint8_t t_skip1a(uint8_t a, uint8_t *p);
uint8_t t_skip1b(uint8_t a, uint8_t *p);
uint8_t t_skip2a(uint8_t a, uint8_t, uint8_t *p);
uint8_t t_skip2b(uint8_t a, uint8_t, uint8_t *p);

uint8_t t_andi(uint8_t a);
uint8_t t_and(uint8_t a, uint8_t b);
uint8_t t_or(uint8_t a, uint8_t b);
uint8_t t_eor(uint8_t a, uint8_t b);

uint8_t t_add(uint8_t a, uint8_t b);
uint8_t t_sub(uint8_t a, uint8_t b);
uint8_t t_subi(uint8_t a);

uint8_t t_ldd(uint8_t a, uint8_t *p);
uint8_t t_std(uint8_t a, uint8_t *p);

int16_t t_cpc(int16_t a, int16_t *p);

uint8_t t_sft1(uint8_t a);
uint8_t t_sft2(uint8_t a);
void t_sft3(uint16_t a, char *r);

void t_mul1a(int16_t in, int16_t *out, int8_t i, uint8_t b);
void t_mul1b(int16_t in, int16_t *out);
int32_t t_mul32(int32_t a, int32_t b);
int32_t t_mul16_r32(int16_t a, int16_t b);
int32_t t_mul16u_r32(uint16_t a, uint16_t b);
int16_t t_mul16(int16_t a, int16_t b);
uint16_t t_mul16u(uint16_t a, uint16_t b);

int8_t t_test_and_set();	// need SUPPORT_EXBI option
int8_t t_test_and_clear();	// need SUPPORT_EXBI option
int8_t t_lock(uint8_t val);
void   t_unlock();

uint8_t buf[4];
int16_t bufi[2] ={ 1, 0 };
int32_t buf32[3];

int main() {
#if 0	
	t_unlock();
	t_lock(0x81);
	t_lock(0x81);
	t_lock(0x80);
	t_unlock();
	t_lock(0x80);
	t_unlock();

	t_test_and_set();
	t_test_and_set();
	t_test_and_clear();
	t_test_and_clear();
	sleep();
#endif
	t_skip1a(0xff, buf);
	t_skip1a(0x00, buf);
	t_skip1b(0xff, buf);
	t_skip1b(0x00, buf);
	t_skip2a(1,1, buf);
	t_skip2a(1,2, buf);
	t_skip2b(1,1, buf);
	t_skip2b(1,2, buf);
	t_and(1, 0);
	t_and(0, 1);
	t_and(1, 1);
	t_and(0, 0);
	t_or(1, 0);
	t_or(0, 1);
	t_or(1, 1);
	t_or(0, 0);
	t_eor(1, 0);
	t_eor(0, 1);
	t_eor(1, 1);
	t_eor(0, 0);
	t_andi(0xff);
	t_andi(0x0f);
	t_and(0xff, 0x80);
	t_and(0x0f, 0x80);
	t_eor(0xff, 0x80);
	t_eor(0x0f, 0x80);

	t_add(0xff, 0x80);
	t_add(0x0f, 0x80);
	t_sub(0xff, 0x80);
	t_sub(0x0f, 0x80);
	t_subi(0xff);
	t_subi(0x0f);

	t_cpc(0, bufi);
	t_cpc(1, bufi);
	t_cpc(2, bufi);
	t_cpc(4, bufi);

	t_sft1(0x56);
	t_sft2(0x56);
	t_sft3(0x1256, (uint8_t *)buf32);

	t_ldd(0xf0, buf);
	t_ldd(0x0f, buf);
	t_std(0xf0, buf);
	t_std(0x0f, buf);

	t_mul1a(0x3bca, bufi, 0, 0x35);
	bufi[0] +=  0x2562;
	nop();
	t_mul1b(0x6801, bufi+1);
	bufi[1] +=  0x2562;

	buf32[0] =  t_mul32(0x1234, 0x5678);
	buf32[1] =  t_mul16u_r32(0x98, 0x76);
	buf32[2] =  t_mul16_r32(0x54, 0x32);

	t_lock(0x81);
	t_lock(0x81);
	t_lock(0x80);
	t_unlock();
	t_lock(0x80);
	t_unlock();

	t_test_and_set();
	t_test_and_set();
	t_test_and_clear();
	t_test_and_clear();
	sleep();
	return 0;
}
